#!/usr/bin/python3
#
#       contRun_couplingSource_fsiChtChtss.py
#
#       OpenFOAMが出力した圧力と熱流束から、FrontISTRを実行して、
#       圧力による変位、熱流束による温度と熱ひずみを計算し、
#       変位と温度を出力する。
#
#   23/10/05    新規作成
#      11/14    fsiCommonConcurrentモジュール使用に変更
#               chtCommonConcurrentモジュール使用に変更
#               chtssCommonConcurrentモジュール使用に変更
#      11/16    allCommonConcAltnモジュール使用に変更
#

import sys, os
import pickle
import shutil
import pyCoupling
import fsiCommonConcurrent as fsiComm
import chtCommonConcurrent as chtComm
import chtssCommonConcurrent as chtssComm
import allCommonConcAltn as allComm

waitCount = 5000
waitTime = 0.01


#----------------
#  coupling_fsi
#----------------
def coupling_fsi():
    """ OpenFOAMが出力した圧力から、FrontISTRが変位を計算する。
    変位を取得するところまで実行する"""
    global dispDict
    patchName = couplingDict["mappingPatch"]
    timeName = couplingDict["timeName"]
    procNo = couplingDict["procNo"]
    solidCaseDir = couplingDict["solidCaseDir"]
    dataDir = fluidCaseDir + "/coupling_FrontISTR/data"
    endStep = 0
    #--- 圧力をmapping ------
    #ptcDataの読み込み
    pressFile = dataDir + "/" + patchName + "_facePress_" + timeName + "_" + procNo + "tm"
    #圧力の点群ファイルを作成
    fsiComm.createPressPtcFile(couplingDict, pressFile)
    if int(procNo) == 0:
        #圧力をmappingしてcntファイルに反映
        fsiComm.createMappingPressFile(couplingDict)
        #--- fistr実行 ----------
        #fsi計算前の設定
        setFsiChtCntFile("fsi")
        endStep, endTime = fsiComm.setDeltaTInCnt_fsi(couplingDict)
        #Fistr起動
        runFistr("fsi", endStep, endTime)
        #--- 変位を取得 ---------
        np = pyCoupling.getNumProcsFistrFromDat(solidCaseDir)
        dispDict = fsiComm.getDispDictFromResFile_fsi(couplingDict, endStep, np)
        #--- 結果file整理（file削除、restart設定）-----
        fsiComm.deleteResSetRestartFileFsi(couplingDict, np)
    return "", endStep

#---------------------
#  coupling_chtChtss
#---------------------
def coupling_chtChtss():
    """ OpenFOAMが出力した熱流束から、FrontISTRが温度を計算する。"""
    solidCaseDir = couplingDict["solidCaseDir"]
    fluidCaseDir = couplingDict["fluidCaseDir"]
    patchName = couplingDict["mappingPatch"]
    timeName = couplingDict["timeName"]
    procNo = couplingDict["procNo"]
    dataDir = fluidCaseDir + "/coupling_FrontISTR/data"
    #---- heatFluxをmapping ---------
    #heatFluxの点群ファイル作成
    htFile = dataDir + "/" + patchName + "_faceHeatFlux_" + timeName + "_" + procNo + "tm"
    chtComm.createHeatFluxPtcFile(couplingDict, htFile)
    if int(procNo) == 0:
        #heatFluxをmappingしてcntファイルに反映
        chtComm.createMappingHeatFlux(couplingDict)
        #---- fistr実行（温度計算） --------------
        #cht計算前の設定
        setFsiChtCntFile("cht")
        endStep, endTime = chtComm.setDeltaTInCnt_cht(couplingDict)
        #Fistr起動
        runFistr("cht", endStep, endTime)
        #--- 温度を取得 ---------
        #温度のptcデータを作成
        print("(py) --- creating temperature PTC ...", flush=True)
        np = pyCoupling.getNumProcsFistrFromDat(solidCaseDir)
        tempDict = chtComm.getTempDictFromResFile(couplingDict, endStep, np)
        tempPtc = chtComm.createTempPtc(couplingDict, tempDict)
        #温度をmappingしたfileを作成
        print("(py) --- temperature value is mapping to patch '" + patchName + "' ...", flush=True)
        chtComm.createMappingTempFile(couplingDict, tempPtc, endStep)
        #---- fistr実行（熱ひずみ計算）----------
        #chtss計算前の設定(cnt,mshファイルを設定)
        setFsiChtCntFile("chtss")
        #温度の計算結果をcntファイルに設定
        chtssComm.remakeFistrCntFileForTemperature(couplingDict, tempDict)
        #Fistr起動(endStep, endTimeはdummy)
        runFistr("chtss", endStep, endTime)
        #---- 変位を取得 -----------------------
        #chtssの変位を取得
        npSs = pyCoupling.getNumProcsFistrFromDat(solidCaseDir)
        dispChtssDict = chtssComm.getDispDictFromResFile(couplingDict, endStep, npSs)
        #変位をbinで保存
        savePickleChtssDict(dispChtssDict)
        #--- 結果file整理（file削除、restart設定）----

        chtComm.deleteResSetRestartFileCht(couplingDict, np)
    return ""

#
#  setFsiChtCntFile
def setFsiChtCntFile(sect):
    """ sectに応じたdatファイルに修正"""
    solidCaseDir = couplingDict["solidCaseDir"]
    pasteDatFile = solidCaseDir + os.sep + "hecmw_ctrl.dat"
    if sect == "fsi":
        datFile = solidCaseDir + os.sep + "hecmw_ctrl_fsi.dat"
    elif sect == "cht":
        datFile = solidCaseDir + os.sep + "hecmw_ctrl_cht.dat"
    elif sect == "chtss":
        datFile = solidCaseDir + os.sep + "hecmw_ctrl_chtss.dat"
    shutil.copy(datFile, pasteDatFile)

#
#  runFistr
#----------
def runFistr(sect, endStep, endTime):
    """ FrontISTRを起動"""
    allComm.runFistr(couplingDict, mmCommArea, sect, endStep, endTime)
    return

#
#  savePickleChtssDict
#--------------
def savePickleChtssDict(dispChtssDict):
    """ 変位を「solidCase/dispChtssDict.bin」で保存する"""
    solidCaseDir = couplingDict["solidCaseDir"]
    fileName = solidCaseDir + "/dispChtssDict.bin"
    f = open(fileName, "wb")
    pickle.dump(dispChtssDict, f)
    f.close()

#
#  loadPickleChtssDict
#----------------------
def loadPickleChtssDict():
    """ 「solidCase/dispChtssDict.bin」から辞書をloadする"""
    global dispDict
    solidCaseDir = couplingDict["solidCaseDir"]
    fileName = solidCaseDir + "/dispChtssDict.bin"
    f = open(fileName, "rb")
    dispChtssDict = pickle.load(f)
    f.close()
    return dispChtssDict

#------------------------
#  createPtcMappingFile
#------------------------
def createPtcMappingFile(endStep):
    """ fsiとchtssの変位を加算し、ptc、mappingファイルを作成する"""
    global dispDict
    procNo = couplingDict["procNo"]
    if int(procNo) == 0:
        #fsi, chtss変位を加算
        dispChtssDict = loadPickleChtssDict()
        for key in dispDict.keys():
            dispDict[key][0] += dispChtssDict[key][0]
            dispDict[key][1] += dispChtssDict[key][1]
            dispDict[key][2] += dispChtssDict[key][2]
        #変位のptcを作成
        print("(py) --- creating displacement PTC ...", flush=True)
        solidCaseDir = couplingDict["solidCaseDir"]
        nodeDictName = solidCaseDir + "/nodeDict.bin"
        f = open(nodeDictName, "rb")
        nodeDict = pickle.load(f)
        f.close()
        dispPtc = []
        for nodeNo in nodeDict.keys():
            disp = dispDict[nodeNo]
            loc = nodeDict[nodeNo]
            dispPtc.append([loc, disp])
        #mapping
        patchName = couplingDict["mappingPatch"]
        print("(py) --- displacement value is mapping to patch '" + patchName + "' ...", flush=True)
        fsiComm.createMappingDispFile(couplingDict, dispPtc, endStep)

#------------------
#  deleteDispFile
#------------------
def deleteDispFile():
    """ 不要なdispFileを削除"""
    #不要なfileを削除--------------
    timeName = couplingDict["timeName"]
    deltaT = couplingDict["deltaT"]
    writeInterval = couplingDict["writeInterval"]
    patchName = couplingDict["mappingPatch"]
    fluidCaseDir = couplingDict["fluidCaseDir"]
    timeIndex = couplingDict["timeIndex"]
    procNo = couplingDict["procNo"]
    dataDir = fluidCaseDir + "/coupling_FrontISTR/data" 
    name = patchName + "_disp_" + timeIndex + "_" + procNo + "tm"
    orgFile = dataDir + "/" + name
    #  writeInterval？
    if pyCoupling.isWriteInterval(timeName, writeInterval, deltaT) == True:
        #orgFileをrenameする
        newFile = orgFile[:-2]
        if os.path.exists(orgFile) == True:
            shutil.move(orgFile, newFile)
    else:
        #削除する
        if os.path.exists(orgFile) == True:
            os.remove(orgFile)


if __name__ == "__main__":
    import gettext
    gettext.install("app") # replace with the appropriate catalog name
    _ = gettext.gettext

    currTime = sys.argv[1]      #currTime
    beforeTime = sys.argv[2]    #前回のtime
    procNo = sys.argv[3]        #procNo
    nProcs = sys.argv[4]        #nProcs
    timeIndex = sys.argv[5]     #timeIndex
    fluidCaseDir = os.getcwd()
    couplingData = pyCoupling.couplingData(fluidCaseDir)
    couplingDict = couplingData.read()
    couplingDict["fluidCaseDir"] = fluidCaseDir
    couplingDict["timeName"] = currTime
    couplingDict["beforeTime"] = beforeTime
    couplingDict["procNo"] = procNo
    couplingDict["nProcs"] = nProcs
    couplingDict["timeIndex"] = timeIndex
    waitCount = int(couplingDict["maxWaitTime"])
    #共有メモリの設定
    mmCommFile = allComm.createCommonArea(couplingDict)
    mmCommArea = allComm.mmCommonArea(couplingDict, mmCommFile)
    #現在時間が終了時間？
    if float(couplingDict["timeName"]) >= float(couplingDict["endTime"]):
        exit()
    #----flagをチェック --------------
    if int(procNo) == 0:
        #busy状態？（自身が起動していれば、終了まで待つ）
        mmCommArea.waitUntilFreeCommonArea()
        #setBusy（busyをセット）
        mmCommArea.writeCommonArea("running python\n")
    #----変位を算出 --------------
    dispDict = {}
    #変位のcoupling？
    if pyCoupling.isDispCoupling(couplingDict) == True:
        #fsiでdispDictを取得
        stat, endStep = coupling_fsi()
        if stat != "":
            raise NameError(stat)
    #温度のcoupling？
    if pyCoupling.isTempCoupling(couplingDict) == True:
        #温度熱ひずみ計算。chtss変位は、binで出力
        stat = coupling_chtChtss()
        if stat != "":
            raise NameError(stat)
    #--- 変位を加算して、ptc作成、mapping
    if pyCoupling.isDispCoupling(couplingDict) == True:
        #mapping
        createPtcMappingFile(endStep)
        #dispFileを削除
        deleteDispFile()
    #---- flagをクリア -------------
    if int(procNo) == 0:
        #clearBusy（freeをセット）
        mmCommArea.writeCommonArea("")

        

